package net.javainthebox.heapindicator;

import java.util.HashMap;
import java.util.Map;

import com.sun.tools.visualvm.application.Application;
import com.sun.tools.visualvm.application.views.ApplicationViewsSupport;
import com.sun.tools.visualvm.core.ui.DataSourceViewPlugin;
import com.sun.tools.visualvm.core.ui.DataSourceViewPluginProvider;
import com.sun.tools.visualvm.core.ui.PluggableDataSourceViewProvider;

public class HeapViewProvider
                extends DataSourceViewPluginProvider<Application> {
    private static Map<String, HeapViewProvider> instances
                        = new HashMap<String, HeapViewProvider>();
    
    private String poolName;
    private int index;

    private HeapViewProvider(String poolName, int index) {
        this.poolName = poolName;
        this.index = index;
    }

    // プラグインの生成
    @Override
    protected DataSourceViewPlugin createPlugin(Application application) {
        return new HeapView(application, poolName, index);
    }

    // サポートするデータソース
    @Override
    protected boolean supportsPluginFor(Application application) {
        return true;
    }

    @SuppressWarnings("unchecked")
    static void initialize(String poolName, int index) {
        HeapViewProvider provider = new HeapViewProvider(poolName, index);
        instances.put(poolName, provider);

        // 1. アプリケーション・ビューのサポートオブジェクトの取得
        ApplicationViewsSupport viewSupport = ApplicationViewsSupport.sharedInstance();

        // 2. [監視]タブのプロバイダを取得
        PluggableDataSourceViewProvider viewProvider = viewSupport.getMonitorView();

        // 3. [監視]タブにプラグインのプロバイダを登録
        viewProvider.registerPluginProvider(provider);
    }

    @SuppressWarnings("unchecked")
    static void uninitialize(String poolName) {
        HeapViewProvider provider = instances.remove(poolName);
        
        // [監視]タブからプラグインのプロバイダを削除
        ApplicationViewsSupport viewSupport = ApplicationViewsSupport.sharedInstance();
        PluggableDataSourceViewProvider viewProvider = viewSupport.getMonitorView();
        viewProvider.unregisterPluginProvider(provider);
    }
}
